/* ----------------------------------------------------------------------
   The SINUS Firewall -- a TCP/IP packet filter for Linux

   Written within the SINUS project at the University of Zurich,
   SWITCH, Telekurs Payserv AG, ETH Zurich.
   originally based on the sf Firewall Software (C) 1996 by Robert
   Muchsel and Roland Schmid.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   SINUS Firewall resources:
   SINUS Homepage: http://www.ifi.unizh.ch/ikm/SINUS/
   Firewall Homepage: http://www.ifi.unizh.ch/ikm/SINUS/firewall.html
   Frequently asked questions: http://www.ifi.unizh.ch/ikm/SINUS/sf_faq.html
   Mailing list for comments, questions, bug reports: firewall@ifi.unizh.ch
   ----------------------------------------------------------------------  */


/*
 * Communication with Configuration Server and Client
 *
 */

#ifndef _SF_RELAY_H
#define _SF_RELAY_H

/*#define RELAY_DEBUG*/

#ifndef MAX
#define MAX(a, b)   ((a) > (b) ? (a) : (b))
#endif

struct auth_data {
  char keynum[4];
  char kspace;
  char key[32];
  char lspace;
  char login[20];
  char cr;
};

struct serv_data {
  unsigned long magic;
  unsigned long command;
  unsigned long length;
  unsigned char data[0];
};

#define BUFFER_SIZE		4096
#define MDKEY_SIZE		32
#define LOGIN_MAXLEN		20
#define SERV_MAGIC		0xbedd1bedL
#define SERV_MAXLEN		BUFFER_SIZE - sizeof(struct serv_data)
#define CLIBUF_SIZE		1024
#define CLI_MAGIC		0xbadcafeL
#define CLI_MAXLEN		CLIBUF_SIZE - sizeof(struct serv_data)
#define MAX_TEMPFILES		20

/* server commands */
#define SERV_SERVER_HELLO	0x01L
#define SERV_CLIENT_HELLO	0x02L
#define SERV_CLOSE		0x03L
#define SERV_CLOSE_ACK		0x04L
#define SERV_PING		0x05L
#define SERV_PING_REPLY		0x06L
#define SERV_GET_FILE		0x07L
#define SERV_FILE_DATA		0x08L
#define SERV_WRITE_FILE		0x09L
#define SERV_WRITE_OK		0x0AL
#define SERV_WRITE_FAIL		0x0BL
#define SERV_READ_ADDR		0x0CL
#define SERV_READ_RULES		0x0DL
#define SERV_READ_VARS		0x0EL
#define SERV_RECONFIG           0x0FL
#define SERV_CHECKCONFIG        0X10L
#define SERV_OK                 0X11L
#define SERV_ERROR              0x12L
#define SERV_READ_TCP		0x13L
#define SERV_KILL_TCP           0x14L
#define SERV_RESET_USAGE        0x15L
#define SERV_AUTHENTICATE       0x16L
#define SERV_ERROR_AUTH	       0x17L
#define SERV_NOTAUTHENTICATED	  0x18L
#define SERV_SERVER_AUTHOK	    0x19L
#define SERV_CLIENT_AUTHOK	    0x1AL
#define SERV_KEY_EXPIRED	    0x1BL
#define SERV_KEY_UPDATE		    0x1CL

/* states */
#define STATE_NOCONN            0x00    /* socket not connected */
#define STATE_CONN              0x01    /* connection established */
#define STATE_CHILD             0x02    /* child is using connection */
#define STATE_USE_MASK          0x03    /* mask: server or child reading? */
#define STATE_WAITHELLO         0x04    /* waiting for hello */
#define STATE_WAITDATA          0x08    /* waiting for more data */
#define STATE_WAITPING          0x10    /* waiting for ping reply */
#define STATE_CLOSEWAIT         0x20    /* waiting for close ack */
#define STATE_RECEIVE_FILE	0x40	/* waiting for more file data */
#define STATE_RECEIVE_TMPFILE	0x80	/* waiting for more file data */
#define STATE_WRITE_FILE	0x100	/* writing file to server */
#define STATE_NOTAUTHENTICATED	0x200	/* connection not yet authenticated */

#endif
