/* ----------------------------------------------------------------------
   The SINUS Firewall -- a TCP/IP packet filter for Linux

   Written within the SINUS project at the University of Zurich,
   SWITCH, Telekurs Payserv AG, ETH Zurich.
   originally based on the sf Firewall Software (C) 1996 by Robert
   Muchsel and Roland Schmid.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   SINUS Firewall resources:
   SINUS Homepage: http://www.ifi.unizh.ch/ikm/SINUS/
   Firewall Homepage: http://www.ifi.unizh.ch/ikm/SINUS/firewall.html
   Frequently asked questions: http://www.ifi.unizh.ch/ikm/SINUS/sf_faq.html
   Mailing list for comments, questions, bug reports: firewall@ifi.unizh.ch
   ----------------------------------------------------------------------  */


/*
 * Routines to send TCP reset for the SF firewall packet filter
 * 
 * $Id: sf_icmp.c,v 1.0 1996/08/28 14:37:44 robby Rel $
 *
 */

/* This code has been rewritten for Kernel 2.2.x. It's based on the rewritten 
 * send_tcp procedure from the sf_tcp.c. I have not tested this code very well, 
 * but on my private network it works. 
 *
 * Known problems:
 *   - local reply dont work; at example you cannot ping firewall interfaces 
 *     from the firewall it self by using this reply code. If realy nessasary
 *     you can work around by accepting this packages and let the reply done by
 *     the original Kernel code. 
 *   - the ping reply dont work with big (> 576 bytes) ping datablocks because
 *     it is not able to fragment the replying package.
 *
 * If you have problems not listed here please contact me at Hessi@iname.com
 *                                                   Marcel Hesselbarth 10/4/00
 */


#include <linux/config.h>
#ifdef CONFIG_MODVERSIONS
#define MODVERSIONS 1
#endif
#include <linux/modversions.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/errno.h>
#include <linux/net.h>
#include <linux/in.h>
#include <linux/inet.h>
#include <linux/netdevice.h>
#include <linux/icmp.h>
#include <linux/tcp.h>
#include <linux/udp.h>
#include <linux/ip.h>
#include <linux/malloc.h>
#include <linux/timer.h>
#include <net/checksum.h>
#include <asm/checksum.h>
#include "sf_kernel.h"
#include "sf_rc.h"
#include "sf_global.h"
#include "sf_filter.h"
#include "sf_control.h"
#include "sf_icmp.h"
#include "sf_ip.h"

void icmp_send_sf(struct device *dev, struct iphdr *ip, struct icmphdr *icmp, int icmpsize)
{
  struct sk_buff *skb;
  struct rtable *rt;
  struct icmphdr *icmp2;
  __u32 saddr, daddr;
  register const __u16 *sp;
  register __u32 csum;
  int st, len = sizeof(struct iphdr) + icmpsize;
 
#ifdef SF_DEBUG_ICMP
  printk("reply to type %d from %d to %d size %d\n",
         ip->protocol,ip->saddr,ip->daddr,ntohs(ip->tot_len));
#endif
  
  /* create and Initializing sk-buffer */
  skb = dev_alloc_skb(len); 
  if (skb == NULL)
    return;
  skb->nh.iph = (struct iphdr *) skb_put(skb, sizeof(struct iphdr));
  skb->h.icmph = (struct icmphdr *) skb_put(skb, icmpsize);
  rt = (struct rtable *) skb->dst;
  skb->dev = dev;
  skb->sk = NULL;
  skb->pkt_type = PACKET_OTHERHOST;
  skb->csum = 0;
  skb->protocol = __constant_htons(ETH_P_IP);


  /* Generating IP-header */
  saddr = ip->daddr;  /* SWAP! */
  daddr = ip->saddr;
  memcpy(skb->nh.iph, ip, sizeof(struct iphdr));
  skb->nh.iph->saddr = saddr;
  skb->nh.iph->daddr = daddr;
  skb->nh.iph->protocol = IPPROTO_ICMP;
  skb->nh.iph->ttl = 255;
  skb->nh.iph->tot_len = htons(len);
  skb->nh.iph->frag_off = 0;
  skb->nh.iph->ihl = sizeof(struct iphdr) >> 2;
  skb->nh.iph->version = IPVERSION;
  skb->nh.iph->check = 0;
  skb->nh.iph->check = ip_fast_csum((char *) skb->nh.iph, skb->nh.iph->ihl);

   
  /* Calculating route... */
  if ((st = ip_route_output(&rt, daddr, 0, 0, 0)) != 0) {
#ifdef SF_DEBUG_ICMP
    printk(KERN_ALERT "sf: icmp_send_sf: Calculating routes failed! errno: %d",
           st);
#endif
    kfree_skb(skb);
    return;
  }
  skb->dst = &rt->u.dst;


  /* copy ICMP header */
  icmp2 = skb->h.icmph;
  memcpy(icmp2, icmp, icmpsize);
  icmp2->checksum = 0;
  

  /* Calculate ICMP checksum */
  csum = 0;
  sp = (__u16 *) icmp2;
  while (icmpsize > 1) {
    csum += *sp++;
    icmpsize -= 2; }
  if (icmpsize == 1) 
    csum += *sp & 0x00ff;
  csum = (csum & 0xffff) + (csum >> 16);
  csum += (csum >> 16);
  icmp2->checksum = ~csum;
  

  /* And send the packet... */
#ifdef SF_DEBUG_ICMP
  printk("Send ICMP type %d:%d from %d to %d size %d\n",
         icmp2->type,icmp2->code,saddr,daddr,len);
#endif
  ip_finish_output(skb);
}





void icmp_reflect(struct device *dev, struct iphdr *ip, struct icmphdr *icmp)
{
  /* answer to ping */
  int iplen = ntohs(ip->tot_len);

#ifdef SF_DEBUG
  printk("icmp_reflect");
#endif

  if (icmp->type != ICMP_ECHO)	   /* Cannot reply */
    return;

  if (iplen > 576)                 /* Cannot reply to fragmented ping packages */
    return;

  /* generating ICMP header */
  icmp->type = ICMP_ECHOREPLY;     /* use reseved packet to reply */

  icmp_send_sf(dev,ip,icmp,iplen - (ip->ihl << 2));    /* icmpsize = total size - ip header */
}



void send_icmp_error(struct device *dev, struct iphdr *ip, int type, int code)
{
  /* send ICMP error */
  char icmpdata[576 - sizeof(struct iphdr)];
  struct icmphdr *icmp = (struct icmphdr*) &icmpdata;
  int icmpsize = sizeof(struct icmphdr) + ntohs(ip->tot_len);
  
#ifdef SF_DEBUG
  printk("send_icmp_error\n");
#endif

  if (ip->protocol == IPPROTO_ICMP)	   	/* Replying errors with errors is not allowed! */
    return;

  /* generating ICMP header */
  if (icmpsize > 576 - sizeof(struct iphdr))	/* reply max 576 bytes of reseaved package */
    icmpsize = 576 - sizeof(struct iphdr);	/* (www standart for error reply)*/
  icmp->type = type;
  icmp->code = code;
  icmp->un.gateway = ip->daddr;			/* ip of replying host (optional) */
  memcpy(&(icmpdata[sizeof(struct icmphdr)]), ip, icmpsize - sizeof(struct icmphdr));  /* reply old massage */
  
  icmp_send_sf(dev,ip,icmp,icmpsize);
}
